function [x,relres,iter,flag]=conjgrad(A,b,x,P,nmax,tol)
% CONJGRAD metodo del gradiente coniugato.
%  [X,RELRES,ITER,FLAG]=CONJGRAD(A,B,X,P,NMAX,TOL) risolve
%  il sistema A*X=B con il metodo del gradiente coniugato. TOL specifica la
%  tolleranza per il metodo. NMAX indica il numero massimo di iterazioni.
%  X e` il vettore iniziale. P e` il precondizionatore, se P e` l'identita`,
%  porre P=[]. RELRES e` il residuo normalizzato. Se FLAG e` = 1, allora
%  RELRES > TOL. ITER e` l'iterazione alla quale la soluzione X e` calcolata.
[n,m]=size(A);
if n ~= m, error('Solo sistemi quadrati'); end
flag = 0; iter = 0; bnrm2 = norm( b );
if bnrm2==0
x=zeros(n,1); relres=0; return
end
r=b-A*x; relres=norm(r)/bnrm2;
while relres > tol && iter< nmax
    if ~isempty(P), z=P\r; else z=r; end
    rho=r'*z;
    if iter > 1
        beta=rho/rho1;
        p=z+beta*p;
    else
        p=z;
    end
    q=A*p;
    alpha=rho/(p'*q);
    x=x+alpha*p;
    r=r-alpha*q;
    relres=norm(r)/bnrm2;
    iter=iter+1;
    rho1 = rho;
end
if relres > tol, flag = 1; end
return
