function [xn,iter,root,itrefin]=mulldefl(A,n,tol,x0,x1,x2,nmax,iref)
% MULLDEFL metodo di Muller con raffinamento.
%  [XN,ITER,ROOT,ITREFIN]=MULLDEFL(A,N,TOL,X0,X1,X2,NMAX,IREF) calcola
%  tutti gli zeri di un polinomio di grado N avente coefficienti
%  A(1),...,A(N). TOL specifica la tolleranza del metodo e X0 e` un dato
%  iniziale. NMAX specifica il numero massimo di iterazioni. Se IFLAG e`
%  uguale a 1, viene attivata la procedura di raffinamento.
apoly=A;
if iref==0, itrefin=[]; end
root=zeros(n,1);
for i=1:n
    xn(1,i)=x0; xn(2,i)=x1; xn(3,i)=x2;
    it=0; err=tol+1; k=2; Ndeg=n-i+1;
    if Ndeg==1
        it=it+1; k=0; xn(it,i)=-A(2)/A(1);
    else
        while err > tol && it < nmax
            k=k+1; it=it+1;
            [f0,B]=horner(A,Ndeg,xn(k-2,i)); [f1,B]=horner(A,Ndeg,xn(k-1,i));
            [f2,B]=horner(A,Ndeg,xn(k,i));
            f01=(f1-f0)/(xn(k-1,i)-xn(k-2,i));  f12=(f2-f1)/(xn(k,i)-xn(k-1,i));
            f012=(f12-f01)/(xn(k,i)-xn(k-2,i));
            w=f12+(xn(k,i)-xn(k-1,i))*f012;
            arg=w^2-4*f2*f012; d1=w-sqrt(arg);
            d2=w+sqrt(arg); den=max(d1,d2);
            if den~=0
                xn(k+1,i)=xn(k,i)-(2*f2)/den;
                err=abs(xn(k+1,i)-xn(k,i));
            else
                fprintf(' Arresto causa annullamento denominatore ');
                return
            end
        end
    end
    radix=xn(k+1,i);
    if iref==1
        alfa=radix; itr=1; err=tol+1;
        while err > tol*1e-3 && itr < nmax
            [px,B]=horner(apoly,n,alfa); [pdx,C]=horner(B,n-1,alfa);
            if pdx == 0
                fprintf(' Arresto causa annullamento derivata '); err=0;
            end
            itr=itr+1;
            if pdx~=0
                alfa2=alfa-px/pdx; err=abs(alfa2-alfa); alfa=alfa2;
            end
        end
        itrefin(i)=itr-1; xn(k+1,i)=alfa; radix=alfa;
    end
    iter(i)=it; root(i)=radix; [px,B]=horner(A,Ndeg-1,xn(k+1,i)); A=B;
end
return
