function [zero,iter,xvect,xdif,fx]=regfalsi(fun,xm1,x0,tol,nmax)
% REGFALSI metodo Regula Falsi.
% [XVECT,XDIF,FX,NIT]=REGFALSI(XM1,X0,TOL,NMAX,FUN) cerca lo zero
% di una funzione continua FUN usando il metodo Regula Falsi partendo
% da X0 e XM1. TOL e NMAX specificano la tolleranza ed il massimo numero di itera-
% zioni. FUN riceve in ingresso lo scalare x e restituisce un valore scalare reale.
% ZERO e` l'approssimazione della radice, ITER e` il numero di iterate svolte.
% XVECT e` il vettore delle iterate, XDIF il vettore delle differenze tra iterate
% successive, FX il vettore dei residui. nit=0;
fxm1=fun(xm1); fx0=fun(x0); fx=[fxm1;fx0];
if fxm1*fx0 >=0
    warning('Attenzione: fun(xm1)*fun(x0)>=0')
    zero=[]; iter=[]; xvect=[]; xdif=[]; fx=[];
    return
end
xvect=[xm1;x0]; xdif=[]; f=tol+1; kprime=1;
iter=0; xk=x0; fxk=fx0;
while iter < nmax && abs(fxk) > tol
    i=length(xvect);
    while i >= kprime
        i=i-1; fxkpr=fun(xvect(i));
        if fxkpr*fxk < 0
            xkpr=xvect(i); kprime=i; break;
        end
    end
    x=xk-fxk*(xk-xkpr)/(fxk-fxkpr);
    xk=x; fxk=fun(x);
    xvect=[xvect; xk]; fx=[fx; fxk];
    err=abs(xk-xkpr); xdif=[xdif; err]; iter=iter+1;

end
zero=xvect(end);
return
